/*
  Keyboard Controller Example

  Shows the output of a USB Keyboard connected to
  the Native USB port on an Arduino Zero

  created 8 Oct 2012
  by Cristian Maglie

  http://arduino.cc/en/Tutorial/KeyboardController

  This sample code is part of the public domain.
*/
// 5/5/2019 KMatocha - added special treatement for:
//      - direction arrow keys (up down left right)
//      - page up and page down (pgup, pgdown)
//
// 5/11/2019 KMatocha - added long press handling of keyDown

// Require keyboard control library
#include <KeyboardController.h>

// On SAMD boards where the native USB port is also the serial console, use
// Serial1 for the serial console. This applies to all SAMD boards except for
// Arduino Zero and M0 boards.
#if (USB_VID==0x2341 && defined(ARDUINO_SAMD_ZERO)) || (USB_VID==0x2a03 && defined(ARDUINO_SAM_ZERO))
#define SerialDebug SERIAL_PORT_MONITOR
#else
#define SerialDebug Serial1
#endif

unsigned long pressTime = 0; // Stores the time when a key is pressed
unsigned long nextPrintTime = 0; // Stores the time when a key is pressed
int numKeysPressed = 0; // keeps track of how many keys are currently pressed

// Settings for the keyboard repeat
unsigned long firstWait = 450; // wait 450 ms after first press to repeat a keypress
unsigned long secondWait = 115; // wait 85 ms for subsequent key press repeats

int lastKeyPressed;

// Initialize USB Controller
USBHost usb;

// Attach keyboard controller to USB
KeyboardController keyboard(usb);

void printKey();

// This function intercepts key press
void keyPressed() {
  //SerialDebug.print("Pressed:  ");
  nextPrintTime = millis() + firstWait;
  lastKeyPressed = keyboard.getOemKey();
  printKey();
  numKeysPressed = numKeysPressed + 1;
}



// This function intercepts key release
void keyReleased() {
  //SerialDebug.print("Released: ");
  //printKey();
  //keyIsPressed = false; // a key was released
  numKeysPressed = numKeysPressed - 1;
}

void printKey() {
  //SerialDebug.write(keyboard.getKey());  //This ** was used on
  //                                        5/4/2020 - and is partially working, but not the arrow keys.

  switch (keyboard.getOemKey()) { // Trial on 5/4/2020 to see if it will fix arrow keys

    case 80:
      SerialDebug.write(27); // left
      SerialDebug.write(91);
      SerialDebug.write(68);
      break;

    case 79:
      SerialDebug.write(27); // Right
      SerialDebug.write(91);
      SerialDebug.write(67);
      break;

    case 81:
      SerialDebug.write(27); // down
      SerialDebug.write(91);
      SerialDebug.write(66);
      break;

    case 82:
      SerialDebug.write(27); // up
      SerialDebug.write(91);
      SerialDebug.write(65);
      break;


    case 74: //home
      SerialDebug.write(27);
      SerialDebug.write(91);
      SerialDebug.write(72);
      break;

    case 77: //end
      SerialDebug.write(27);
      SerialDebug.write(91);
      SerialDebug.write(70);
      break;

    case 75: //page up
      SerialDebug.write(27);
      SerialDebug.write(91);
      SerialDebug.write(53);
      SerialDebug.write(126);
      break;

    case 78: //page down
      SerialDebug.write(27);
      SerialDebug.write(91);
      SerialDebug.write(54);
      SerialDebug.write(126);
      break;



    default:
      SerialDebug.write(keyboard.getKey());
      //SerialDebug.write(keyboard.getOemKey()); # can use this for debug
      break;

  }
}

uint32_t lastUSBstate = 0;

void setup()
{
  SerialDebug.begin( 115200 );
  //SerialDebug.println("Keyboard Controller Program started");

  if (usb.Init())
    // SerialDebug.println("USB host did not start.");

    delay( 20 );
}

void loop()
{
  // Process USB tasks
  usb.Task();

  //  uint32_t currentUSBstate = usb.getUsbTaskState();
  //  if (lastUSBstate != currentUSBstate) {
  //    SerialDebug.print("USB state changed: 0x");
  //    SerialDebug.print(lastUSBstate, HEX);
  //    SerialDebug.print(" -> 0x");
  //    SerialDebug.println(currentUSBstate, HEX);
  //    switch (currentUSBstate) {
  //      case USB_ATTACHED_SUBSTATE_SETTLE: SerialDebug.println("Device Attached"); break;
  //      case USB_DETACHED_SUBSTATE_WAIT_FOR_DEVICE: SerialDebug.println("Detached, waiting for Device"); break;
  //      case USB_ATTACHED_SUBSTATE_RESET_DEVICE: SerialDebug.println("Resetting Device"); break;
  //      case USB_ATTACHED_SUBSTATE_WAIT_RESET_COMPLETE: SerialDebug.println("Reset complete"); break;
  //      case USB_STATE_CONFIGURING: SerialDebug.println("USB Configuring"); break;
  //      case USB_STATE_RUNNING: SerialDebug.println("USB Running"); break;
  //    }
  //    lastUSBstate = currentUSBstate;
  //  }

  unsigned long currentMillis = millis();
  if ( (keyboard.getOemKey() == lastKeyPressed)) {
    if ( (numKeysPressed == 1) && (currentMillis > nextPrintTime) ) { // it has only printed once.
      nextPrintTime = currentMillis + secondWait;
      printKey();

    }
  }


}
